﻿using System;
using Microsoft.Practices.ObjectBuilder2;
using Microsoft.Practices.Unity;
using Microsoft.Practices.Unity.InterceptionExtension;

namespace Artech.InterceptableUnity
{
    public class ExtendedInstanceInterceptionStrategy : InstanceInterceptionStrategy
    {
        public ExtendedInterception Interception
        { get; private set; }

        public ExtendedInstanceInterceptionStrategy(ExtendedInterception interception)
        {
            if (null == interception)
            {
                throw new ArgumentNullException("interception");
            }

            this.Interception = interception;
        }

        private static IInstanceInterceptionPolicy FindInterceptorPolicy(IBuilderContext context)
        {
            Type buildKey = BuildKey.GetType(context.BuildKey);
            Type type = BuildKey.GetType(context.OriginalBuildKey);
            IInstanceInterceptionPolicy policy = context.Policies.Get<IInstanceInterceptionPolicy>(context.BuildKey, false) ?? context.Policies.Get<IInstanceInterceptionPolicy>(buildKey, false);
            if (policy != null)
            {
                return policy;
            }
            policy = context.Policies.Get<IInstanceInterceptionPolicy>(context.OriginalBuildKey, false) ?? context.Policies.Get<IInstanceInterceptionPolicy>(type, false);
            return policy;
        }

        public override void PreBuildUp(IBuilderContext context)
        {
            if (BuildKey.GetType(context.BuildKey) == typeof(IUnityContainer))
            {
                return;
            }

            IInstanceInterceptionPolicy policy = FindInterceptorPolicy(context);
            if (null != policy)
            {
                if (policy.Interceptor.CanIntercept(BuildKey.GetType(context.BuildKey)))
                {
                    this.Interception.SetDefaultInterceptorFor(BuildKey.GetType(context.BuildKey), policy.Interceptor);
                }
            }
            else
            {
                if (this.Interception.Interceptor.CanIntercept(BuildKey.GetType(context.BuildKey)) && this.Interception.Interceptor is IInstanceInterceptor)
                {
                    this.Interception.SetDefaultInterceptorFor(BuildKey.GetType(context.BuildKey), (IInstanceInterceptor)this.Interception.Interceptor);
                }
            }
            base.PreBuildUp(context);
        }
    }
}