﻿using System;
using System.Configuration;
using Microsoft.Practices.Unity;
using Microsoft.Practices.Unity.Configuration;
using Microsoft.Practices.Unity.InterceptionExtension;

namespace Artech.InterceptableUnity
{
    public class ExtendedInterceptionElement : UnityContainerExtensionConfigurationElement
    {
        [ConfigurationProperty("interceptor", IsRequired = false, DefaultValue = "")]
        public string Interceptor
        {
            get
            {
                return (string)this["interceptor"];
            }
        }

        public override void Configure(IUnityContainer container)
        {
            base.Configure(container);
            ExtendedInterception interception = new ExtendedInterception();
            if (!string.IsNullOrEmpty(this.Interceptor))
            {
                var type = System.Type.GetType(this.Interceptor);
                if (null == type)
                {
                    throw new ConfigurationErrorsException(string.Format("The {0} is not a valid Interceptor.", this.Interceptor));
                }

                if (!typeof(IInterceptor).IsAssignableFrom(type))
                {
                    throw new ConfigurationErrorsException(string.Format("The {0} is not a valid Interceptor.", this.Interceptor));
                }
                interception.Interceptor = (IInterceptor)Activator.CreateInstance(type);
            }

            container.AddExtension(interception);
        }
    }
}
